/**
 * Copyright (C) 2008 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <gtk/gtk.h>
#include <libxfce4util/libxfce4util.h>
#include <libxfcegui4/libxfcegui4.h>
#include <libxfce4panel/xfce-panel-plugin.h>

#include "event-label.h"

typedef struct _ELabel
{
    GtkWidget *ebox;
    struct {
        gboolean show_label;
        gboolean show_image;
        gboolean run_cmd;
        gchar *label_text;
        gchar *image_file;
        gchar *cmd;
    } config;

    XfcePanelPlugin *plugin;
} ELabel;

void elabel_construct (XfcePanelPlugin *plugin);
void elabel_save      (XfcePanelPlugin *plugin, ELabel *elabel);
void update_widget    (ELabel *elabel);

/* register the plugin */
XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL (elabel_construct);

void
elabel_read (ELabel *elabel)
{
    XfceRc *rc;
    gchar *filename;

    filename = xfce_panel_plugin_save_location (elabel->plugin, TRUE);
    if (G_LIKELY (filename))
    {
        rc = xfce_rc_simple_open (filename, TRUE);
        g_free (filename);

        if (G_LIKELY (rc))
        {
            elabel->config.show_label = xfce_rc_read_bool_entry (rc, "show_label", TRUE);
            elabel->config.show_image = xfce_rc_read_bool_entry (rc, "show_image", TRUE);
            elabel->config.run_cmd = xfce_rc_read_bool_entry (rc, "show_cmd", TRUE);
            elabel->config.label_text = g_strdup (xfce_rc_read_entry (rc, "label_text", "Label"));
            elabel->config.image_file = g_strdup (xfce_rc_read_entry (rc, "image_file", ""));
            elabel->config.cmd = g_strdup (xfce_rc_read_entry (rc, "cmd", "xterm"));
            xfce_rc_close (rc);
            return;
        }
    }

    elabel->config.show_label = TRUE;
    elabel->config.show_image = FALSE;
    elabel->config.run_cmd = FALSE;
    elabel->config.label_text = "Label";
    elabel->config.image_file = "";
    elabel->config.cmd = "xterm";
}

void
on_check_btn_toggled (GtkWidget *chk, gpointer data)
{
    gtk_widget_set_sensitive (GTK_WIDGET (data),
        gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (chk)));
}

/* Panel plugin callbacks */
void
elabel_configure (XfcePanelPlugin *plugin,
                  ELabel          *elabel)
{
    GtkWidget *dialog;
    GtkWidget *label_chk, *image_chk, *cmd_chk;
    GtkWidget *label_entry, *image_file_btn;
    GtkWidget *cmd_entry;
    GtkWidget *hbox;
    GtkSizeGroup *l_group, *r_group;

    xfce_panel_plugin_block_menu (plugin);
    dialog = xfce_titled_dialog_new_with_buttons (_("Label Plugin"),
                                                  NULL,
                                                  GTK_DIALOG_DESTROY_WITH_PARENT,
                                                  GTK_STOCK_OK, GTK_RESPONSE_OK,
                                                  NULL);
    gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);

    gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
    l_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
    r_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 0);
    label_chk = gtk_check_button_new_with_label (_("Display a label: "));
    gtk_size_group_add_widget (l_group, label_chk);
    gtk_box_pack_start (GTK_BOX (hbox), label_chk, FALSE, FALSE, 2);
    label_entry = gtk_entry_new ();
    gtk_size_group_add_widget (r_group, label_entry);
    gtk_widget_set_sensitive (label_entry, FALSE);
    gtk_box_pack_start (GTK_BOX (hbox), label_entry, FALSE, FALSE, 2);

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 0);
    image_chk = gtk_check_button_new_with_label (_("Display an image: "));
    gtk_size_group_add_widget (l_group, image_chk);
    gtk_box_pack_start (GTK_BOX (hbox), image_chk, FALSE, FALSE, 2);
    image_file_btn = gtk_file_chooser_button_new (_("Select an image"),
                                                  GTK_FILE_CHOOSER_ACTION_OPEN);
    gtk_size_group_add_widget (r_group, image_file_btn);
    gtk_widget_set_sensitive (image_file_btn, FALSE);
    gtk_box_pack_start (GTK_BOX (hbox), image_file_btn, FALSE, FALSE, 2);

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 0);
    cmd_chk = gtk_check_button_new_with_label (_("Run a command on clic: "));
    gtk_size_group_add_widget (l_group, cmd_chk);
    gtk_box_pack_start (GTK_BOX (hbox), cmd_chk, FALSE, FALSE, 2);
    cmd_entry = gtk_entry_new ();
    gtk_size_group_add_widget (r_group, cmd_entry);
    gtk_widget_set_sensitive (cmd_entry, FALSE);
    gtk_box_pack_start (GTK_BOX (hbox), cmd_entry, FALSE, FALSE, 2);

    g_signal_connect (G_OBJECT (label_chk), "toggled",
        G_CALLBACK (on_check_btn_toggled), label_entry);
    g_signal_connect (G_OBJECT (image_chk), "toggled",
        G_CALLBACK (on_check_btn_toggled), image_file_btn);
    g_signal_connect (G_OBJECT (cmd_chk), "toggled",
        G_CALLBACK (on_check_btn_toggled), cmd_entry);

    gtk_entry_set_text (GTK_ENTRY (label_entry), elabel->config.label_text);
    gtk_entry_set_text (GTK_ENTRY (cmd_entry), elabel->config.cmd);
    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (image_file_btn), elabel->config.image_file);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (label_chk), elabel->config.show_label);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (image_chk), elabel->config.show_image);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cmd_chk), elabel->config.run_cmd);

    gtk_widget_show_all (GTK_DIALOG(dialog)->vbox);

    gtk_dialog_run (GTK_DIALOG (dialog));
    elabel->config.show_label = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (label_chk));
    elabel->config.show_image = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (image_chk));
    elabel->config.run_cmd = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cmd_chk));
    elabel->config.label_text = g_strdup (gtk_entry_get_text (GTK_ENTRY (label_entry)));
    elabel->config.image_file = g_strdup (gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (image_file_btn)));
    elabel->config.cmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (cmd_entry)));
    elabel_save (plugin, elabel);
    gtk_widget_destroy (dialog);
    update_widget (elabel);

    xfce_panel_plugin_unblock_menu (plugin);
}

void
elabel_save (XfcePanelPlugin *plugin,
             ELabel          *elabel)
{
    XfceRc *rc;
    gchar *filename;

    filename = xfce_panel_plugin_save_location (elabel->plugin, TRUE);
    if (G_UNLIKELY (filename))
    {
        rc = xfce_rc_simple_open (filename, FALSE);
        g_free (filename);

        if (G_LIKELY (rc))
        {
            xfce_rc_write_bool_entry (rc, "show_label", elabel->config.show_label);
            xfce_rc_write_bool_entry (rc, "show_image", elabel->config.show_image);
            xfce_rc_write_bool_entry (rc, "run_cmd", elabel->config.run_cmd);
            xfce_rc_write_entry (rc, "label_text", elabel->config.label_text);
            xfce_rc_write_entry (rc, "image_file", elabel->config.image_file);
            xfce_rc_write_entry (rc, "cmd", elabel->config.cmd);

            xfce_rc_close (rc);
        }
    }
}

void
update_widget (ELabel *elabel)
{
    if (elabel->config.show_label && elabel->config.label_text)
    {
        gtk_label_set_text (GTK_LABEL (EVENT_LABEL (elabel->ebox)->label), elabel->config.label_text);
        gtk_widget_show (GTK_WIDGET (EVENT_LABEL (elabel->ebox)->label));
    }  else
        gtk_widget_hide (GTK_WIDGET (EVENT_LABEL (elabel->ebox)->label));

    if (elabel->config.show_image && elabel->config.image_file)
    {
        int size = xfce_panel_plugin_get_size (elabel->plugin) - 3;
        GdkPixbuf *pixbuf =
                    gdk_pixbuf_new_from_file_at_scale (elabel->config.image_file,
                                                   -1, size, TRUE, NULL);
        gtk_image_set_from_pixbuf (GTK_IMAGE (EVENT_LABEL (elabel->ebox)->image), pixbuf);
        gtk_widget_show (GTK_WIDGET (EVENT_LABEL (elabel->ebox)->image));
        g_object_unref (pixbuf);
    } else
        gtk_widget_hide (GTK_WIDGET (EVENT_LABEL (elabel->ebox)->image));
    if (elabel->config.run_cmd && elabel->config.cmd)
        event_label_set_cmd (EVENT_LABEL (elabel->ebox), elabel->config.cmd);
    else
        event_label_set_cmd (EVENT_LABEL (elabel->ebox), NULL);
}

ELabel *
elabel_new (XfcePanelPlugin *plugin)
{
    ELabel *elabel;

    elabel = panel_slice_new0 (ELabel);
    elabel->plugin = plugin;
    elabel_read (elabel);

    elabel->ebox = event_label_new (NULL, NULL);
    event_label_set_cmd (EVENT_LABEL (elabel->ebox), "xterm");
    update_widget (elabel);

    gtk_widget_show (elabel->ebox);
    return elabel;
}

void
elabel_free (XfcePanelPlugin *plugin,
             ELabel          *elabel)
{
    panel_slice_free (ELabel, elabel);
}

void
elabel_orientation_changed (XfcePanelPlugin *plugin,
                            GtkOrientation   orientation,
                            ELabel          *elabel)
{
    /* would be nice to do something at some point */
}

gboolean
elabel_size_changed (XfcePanelPlugin *plugin,
                     gint             size,
                     ELabel          *elabel)
{
    GtkOrientation orientation;
    orientation = xfce_panel_plugin_get_orientation (plugin);

    /* new size for the image */
    if (elabel->config.show_image && elabel->config.image_file)
    {
        GdkPixbuf *pixbuf;
        pixbuf = gdk_pixbuf_new_from_file_at_scale (elabel->config.image_file,
                                                    -1, size, TRUE, NULL);
        gtk_image_set_from_pixbuf (GTK_IMAGE (EVENT_LABEL (elabel->ebox)->image), pixbuf);
        g_object_unref (pixbuf);
    }

    if (orientation == GTK_ORIENTATION_HORIZONTAL)
        gtk_widget_set_size_request (GTK_WIDGET (plugin), -1, size);
    else
        gtk_widget_set_size_request (GTK_WIDGET (plugin), size, -1);

    return TRUE;
}

void
elabel_construct (XfcePanelPlugin *plugin)
{
    xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");

    ELabel *elabel;
    elabel = elabel_new (plugin);
    gtk_container_add (GTK_CONTAINER (plugin), elabel->ebox);
    xfce_panel_plugin_add_action_widget (plugin, elabel->ebox);

    g_signal_connect (G_OBJECT (plugin), "free-data",
                      G_CALLBACK (elabel_free), elabel);
    g_signal_connect (G_OBJECT (plugin), "save",
                      G_CALLBACK (elabel_save), elabel);
    g_signal_connect (G_OBJECT (plugin), "size-changed",
                      G_CALLBACK (elabel_size_changed), elabel);
    g_signal_connect (G_OBJECT (plugin), "orientation-changed",
                      G_CALLBACK (elabel_orientation_changed), elabel);

    xfce_panel_plugin_menu_show_configure (plugin);
    g_signal_connect (G_OBJECT (plugin), "configure-plugin",
                    G_CALLBACK (elabel_configure), elabel);
}

