/**
 * Copyright (C) 2008 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>

#include <libxfcegui4/libxfcegui4.h>

#include "event-label.h"

#define MARGIN 2

struct _EventLabelPrivate
{
    GtkWidget *box; /* could be a static var but we might be able
                     * to change the orientation some day */
    gchar *cmd;
};

static void event_label_class_init (EventLabelClass *klass);
static void event_label_init (EventLabel *elabel);

G_DEFINE_TYPE (EventLabel, event_label, GTK_TYPE_EVENT_BOX);

static void
event_label_class_init (EventLabelClass *klass)
{
    g_type_class_add_private (klass, sizeof (EventLabelPrivate));
}

static gboolean
button_press_callback (GtkWidget *w, GdkEventButton *event, gpointer data)
{
    EventLabel *elabel = (EventLabel *) data;
    if (! elabel->priv->cmd)
        return FALSE;
    if (event->type == GDK_BUTTON_PRESS && event->button == 1)
        xfce_exec (elabel->priv->cmd, FALSE, FALSE, NULL);

    return FALSE;
}

static void
event_label_init (EventLabel *elabel)
{
    elabel->priv = G_TYPE_INSTANCE_GET_PRIVATE (elabel,
                                                TYPE_EVENT_LABEL,
                                                EventLabelPrivate);

    elabel->priv->cmd = NULL;

    elabel->priv->box = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (elabel->priv->box), MARGIN);
    gtk_container_add (GTK_CONTAINER (elabel), elabel->priv->box);

    elabel->image = gtk_image_new ();
    gtk_box_pack_start (GTK_BOX (elabel->priv->box), elabel->image,
                        FALSE, FALSE, MARGIN);

    elabel->label = gtk_label_new (NULL);
    gtk_box_pack_start (GTK_BOX (elabel->priv->box), elabel->label,
                        FALSE, FALSE, MARGIN);

    g_signal_connect (G_OBJECT (elabel), "button_press_event",
                      G_CALLBACK (button_press_callback), elabel);

    gtk_widget_show_all (elabel->priv->box);
}

/* Public API */
GtkWidget *
event_label_new (const gchar *label, const gchar *image_path)
{
    EventLabel *elabel;
    elabel = g_object_new (event_label_get_type (), NULL);

    if (label)
        gtk_label_set_text (GTK_LABEL (elabel->label), label);
    if (image_path)
        gtk_image_set_from_file (GTK_IMAGE (elabel->image), image_path);

    return GTK_WIDGET (elabel);
}

void
event_label_set_cmd (EventLabel *elabel, const gchar *cmd)
{
    if (cmd)
        elabel->priv->cmd = g_strdup (cmd);
    else
        elabel->priv->cmd = NULL;
}

const gchar *
event_label_get_cmd (EventLabel *elabel)
{
    return elabel->priv->cmd;
}
